// List_d.cpp
// Example code for deep copy using copy constructor and overloaded "=" operator
// M. Schukat, 2002
 

// Include header files
#include <iostream.h>
#include <stdlib.h>
#include <math.h>

#include "list_d.h"


// Implementation of COPY Constructor DoubSet::DoubSet(const DoubSet &c)
// Syntax: <ClassName>::<ClassName>(const <ClassName>& <VariableName>)
DoubSet::DoubSet(const DoubSet &m)
{
	// Copy attribute number
	number = m.number;

	// List m is empty
	if (number == 0)
	{
		// Init start pointer
		start = 0;
		return;
	}
	else
	{
		// Get copy of start pointer of m
		DoubElem* m_ptr = m.start;

		// Create local instance z of DoubElem
		DoubElem* z;

		// start will point to new DoubElem
		start = new DoubElem;

		// Copy value of first list element of list m to object
		// start is pointing to.
		start->element = m_ptr->element;

		// z pointer = start pointer. We need this assignment because
		// of the while loop below.
		z = start;

		// We have dealt with the first list element of m, we go to the next one.
		m_ptr = m_ptr->next;

		// As long as m_ptr points to another element (e.g. we are still moving
		// through the list of m)...
		while (0 != m_ptr)
		{
			// z (== start in the first loop) points to a new element
			z->next = new DoubElem;

			// We move on to the new element...
			z = z->next;

			// ... and copy the value of the corresponding list element of m_ptr 
			// (the list element belongs to m) into this new element.
			z->element = m_ptr->element;

			// We set the next pointer to zero to terminate the list.
			z->next = 0;

			// And we continue with the next element of m.
			m_ptr = m_ptr->next;
		}
	}
}


// NEW: Prototype for new (overloaded) "=" operator
// Syntax: void <ClassName>::operator=(const <ClassName>& <VariableName>)
void DoubSet::operator=(const DoubSet& m)
{
	// Copy attribute number
	number = m.number;

	// List m is empty
	if (number == 0)
	{
		// Init start pointer
		start = 0;
		return;
	}
	else
	{
		// Get copy of start pointer of m
		DoubElem* m_ptr = m.start;

		// Create local instance z of DoubElem
		DoubElem* z;

		// start will point to new DoubElem
		start = new DoubElem;

		// Copy value of first list element of list m to object
		// start is pointing to.
		start->element = m_ptr->element;

		// z pointer = start pointer. We need this assignment because
		// of the while loop below.
		z = start;

		// We have dealt with the first list element of m, we go to the next one.
		m_ptr = m_ptr->next;

		// As long as m_ptr points to another element (e.g. we are still moving
		// through the list of m)...
		while (0 != m_ptr)
		{
			// z (== start in the first loop) points to a new element
			z->next = new DoubElem;

			// We move on to the new element...
			z = z->next;

			// ... and copy the value of the corresponding list element of m_ptr 
			// (the list element belongs to m) into this new element.
			z->element = m_ptr->element;

			// We set the next pointer to zero to terminate the list.
			z->next = 0;

			// And we continue with the next element of m.
			m_ptr = m_ptr->next;
		}
	}
}


// Implementation of bool DoubSet::IsElement(double x) - old stuff
bool DoubSet::IsElement(double x) const 
{
	// return if list is empty.
	if (Empty())
	{
		return false;
	}
	else
	{
		DoubElem* tmp = start;

		// Go through list until element is found or list is ended.
		do
		{
			if (fabs(tmp->element - x) < 1.0E-15 * fabs(x))
			{
				return true;
			}
			else
			{
				tmp = tmp->next;
			}
		} while (tmp != 0);
	}
	return false;
}


// Implementation of bool DoubSet::Insert(double x) - old stuff
bool DoubSet::Insert(double x)
{
	// Don't add similar elements twice.
	if (IsElement(x))
	{
		return false;
	}
	else
	{
		// Create new element and link it to list.
		DoubElem* tmp = new DoubElem;
		tmp->element = x;
		tmp->next = start;

		start = tmp;

		number++;

		return true;
	}
}


// Implementation of bool DoubSet::Content(void) - old stuff
void DoubSet::Content(void) const
{
	// Print content of list.
	cout << "{ ";

	if (!Empty())
	{
		DoubElem* tmp = start;

		cout << tmp->element;
		
		while (tmp->next != 0)
		{
			tmp = tmp->next;
			cout << ", " << tmp->element;
		}
	}
	cout << " }" << endl;
}


// Destructor - old stuff
DoubSet::~DoubSet(void)
{
	if (Empty())
	{
		return;
	}
	else
	{
		while (start != 0)
		{
			DoubElem* tmp = start;
			start = start->next;
			delete tmp;
		}
	}
}


// Implementation of void DoubSet::IncrementAllElements(void) - old stuff
// All elemets of set are incremented by one.
void DoubSet::IncrementAllElements(void)
{
	// return if list is empty.
	if (!Empty())
	{
		DoubElem* tmp = start;

		// Go through list until element is found or list is ended.
		do
		{
			// Inrecement element by one.
			tmp->element++;

			// Go to next pointer
			tmp = tmp->next;
		} while (tmp != 0);
	}
}